#ifndef TASK_H
#define TASK_H

#include "gmc-common_global.h"

#include <QObject>
#include <QStringList>
#include "passworddialog.h"

class Thread;
class Element;
class ElementWidget;
class ElementData;
class AbstractTask;

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT TaskDescription
{

    friend class AbstractTask;

public:

    enum TaskStatus { Unknown
                    , Fail
                    , Success
                    , Warning
                    , AboutToBeStop };

    TaskDescription();
    TaskDescription( const TaskDescription & /*taskDescription*/ );

    const TaskDescription & operator=( const TaskDescription & /*taskDescription*/ );

    const QString & description() const;
    void setDescription( const QString & /*description*/ );

    TaskStatus status() const;
    void setStatus( const TaskStatus &status );

    const QString & message() const;
    void setMessage( const QString &message );

protected:

    QString _description;
    TaskStatus _status;
    QString _message;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT AbstractTask
{

public:

    enum SpecialTaskFlags { NoSpecialFlag = 0x0     // ordinary task
                          , StoppableFlag = 0x1     // stoppable flag
                          };

    AbstractTask( ElementData * /*elementData*/ );
    virtual ~AbstractTask();

    qulonglong taskSpecialFlags() const;

    bool taskAboutToBeStopped() const;
    bool setTaskAboutToBeStopped();

    TaskDescription & taskDescription();
    const TaskDescription & taskDescription() const;

    virtual void run();
    virtual void afterRun() = 0;

protected:

    qulonglong _specialFlags;
    volatile bool _taskAboutToBeStopped;

    ElementData *_elementData;
    TaskDescription _taskDescription;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ExportHtmlTask : public AbstractTask
{

public:

    ExportHtmlTask(  ElementData * /*elementData*/ );
    virtual ~ExportHtmlTask();
    void run() override;
    void afterRun() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ElementTask : public AbstractTask
{

public:

    ElementTask(Element * /*element*/, ElementData * /*elementData*/ );
    virtual ~ElementTask();

    Element *element() const;

protected:

    Element *_element;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT CreateTask : public ElementTask
{

public:

    CreateTask( Element * /*element*/, ElementData * /*elementData*/ );
    virtual ~CreateTask();

    void afterRun() override;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ReadContentTask : public ElementTask
{

public:

    ReadContentTask( Element * /*element*/ );
    virtual ~ReadContentTask();

    void afterRun() override;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ReadPropertyHeavyTask : public ElementTask
{

public:

    ReadPropertyHeavyTask( Element * /*element*/ );
    virtual ~ReadPropertyHeavyTask();

    void afterRun() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT UpdateTask : public ElementTask
{

public:

    UpdateTask( Element * /*element*/, ElementData * /*elementData*/ );
    virtual ~UpdateTask();

    void afterRun() override;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT DeleteTask : public ElementTask
{

public:

    DeleteTask( Element * /*element*/ );
    virtual ~DeleteTask();

    void afterRun() override;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ServiceTask : public ElementTask
{

public:

    ServiceTask( Element * /*element*/ );
    virtual ~ServiceTask();

    void run() override;

protected:

    bool _standartSequenceArguments;

    QString _serviceCtlScript;
    QString _serviceCtlAction;
    QStringList _serviceCtlParameters;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ReadPropertyLightServiceTask : public ServiceTask
{

public:

    ReadPropertyLightServiceTask( Element * /*element*/ );
    virtual ~ReadPropertyLightServiceTask();

    void afterRun() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT StartStopServiceTask : public ServiceTask
{

public:

    StartStopServiceTask( Element * /*element*/ );
    virtual ~StartStopServiceTask();

    void afterRun() override;

protected:

    bool _resultTask;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT StartServiceTask : public StartStopServiceTask
{

public:

    StartServiceTask( Element * /*element*/ );
    virtual ~StartServiceTask();
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT StopServiceTask : public StartStopServiceTask
{

public:

    StopServiceTask( Element * /*element*/ );
    virtual ~StopServiceTask();

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT RServiceTask : public ServiceTask
{

public:

    RServiceTask( Element * /*element*/ );
    virtual ~RServiceTask();

    void afterRun() override;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT RestartServiceTask : public RServiceTask
{

public:

    RestartServiceTask( Element * /*element*/ );
    virtual ~RestartServiceTask();

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ReloadServiceTask : public RServiceTask
{

public:

    ReloadServiceTask( Element * /*element*/ );
    virtual ~ReloadServiceTask();

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ControlServiceTask : public ElementTask
{

protected:

    static bool existPackage( const QString & );

public:

    ControlServiceTask( Element * /*element*/ );
    virtual ~ControlServiceTask();

    void run() override;
    void afterRun() override;

protected:

    QString     _user;      // Запустить команду под указанным пользователем (с помощью runuser)
    QString     _package;   // Пакет, в котором находится команда
    QString     _command;
    QStringList _arguments;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ElementWidgetTask : public AbstractTask
{

public:

    ElementWidgetTask( ElementWidget * /*elementWidget*/, ElementData * /*elementData*/ );
    virtual ~ElementWidgetTask();

    ElementWidget *elementWidget() const;

protected:

    ElementWidget *_elementWidget;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ImportTask : public ElementWidgetTask
{

public:

    ImportTask( ElementWidget * /*elementWidget*/, ElementData * /*elementData*/ );
    virtual ~ImportTask();

    void afterRun() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ExportWidgetTask : public ElementWidgetTask
{

public:

    ExportWidgetTask( ElementWidget * /*elementWidget*/, ElementData * /*elementData*/ );
    virtual ~ExportWidgetTask();

    void afterRun() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT PrintWidgetTask : public ElementWidgetTask
{

public:

    PrintWidgetTask( ElementWidget * /*elementWidget*/, ElementData * /*elementData*/ );
    virtual ~PrintWidgetTask();
    void run() override;
    void afterRun() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT DummyTask : public ElementTask
{

public:

    DummyTask(Element * /*element*/, bool /*success*/ = true );
    virtual ~DummyTask();
    void run() override;
    void afterRun() override;

protected:

    bool _success;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT StoppableDummyTask : public ElementTask
{

public:

    StoppableDummyTask( Element * /*element*/ );
    virtual ~StoppableDummyTask();
    void run() override;
    void afterRun() override;

};

#endif // RUNNABLE_H
